StandardSystemView subclass:#Calendar
       instanceVariableNames:'calendarView list year'
       classVariableNames:''
       poolDictionaries:''
       category:'Demos-Tutorial'
!

Calendar comment:'
 shows a calendar, like motif example in oReilly 

 start with:
	Calendar start
'!

!Calendar class methodsFor:'documentation'!

documentation
"
    shows a calendar, like the motif example in oReilly. 
    This example shows how to read the output of some
    Unix command via a PipeStream and put it into a textview.
    Also, it includes a simple popupmenu to choose the year.

    start with:
	Calendar open
"
! !

!Calendar methodsFor:'initialization'!

initialize
    |v cView|

    super initialize.

    year := Date today year.

    "
     the label, which will hold the output of the unix cal command
    "
    calendarView := Label in:self. 
    calendarView adjust:#left.
    calendarView font:(Font family:'courier'
			      face:'bold'
			     style:'roman'
			      size:14).

    "adjust my size, to make 8 lines fit into label"
    calendarView label:#(' '
			 ' '
			 ' '
			 ' '
			 ' '
			 ' '
			 ' '
			 ' ').
    style == #st80 ifTrue:[
	calendarView level:1
    ].
    calendarView sizeFixed:true.

    self extent:500 @ (calendarView height + 
		      (device verticalPixelPerMillimeter * 10)) rounded.
    calendarView origin:0.0@0.0 corner:0.66@1.0.

    v := ScrollableView for:SelectionInListView in:self.
    v origin:0.66@0.0 corner:1.0@1.0.
    list := v scrolledView.
    list list:#('January' 'February' 'March'
		'April' 'May' 'June' 'July'
		'August' 'September' 'October'
		'November' 'December').

    list action:[:line | self monthSelection:line].
    list middleButtonMenu:(PopUpMenu
			 labels:(Array with:(year - 2) printString
				       with:(year - 1) printString
				       with:(year    ) printString
				       with:(year + 1) printString
				       with:'other ...')
		      selectors:#(changeYear:
				  changeYear:
				  changeYear:
				  changeYear:
				  otherYear)
			   args:(Array with:(year - 2)
				       with:(year - 1)
				       with:(year    )
				       with:(year + 1)
				       with:nil)
		       receiver:self).

    self label:'Calendar'.
    list selection:(Date today month).
    self monthSelection:(Date today month).
! !

!Calendar methodsFor:'user interaction'!

monthSelection:aMonthNumber
    "change month action from selection list"

    |command p text|

    command := 'cal ' , aMonthNumber printString , ' ' 
		      , year printString.
    p := PipeStream readingFrom:command.
    text := p contents.
    "remove blank line at end"
    (text at:(text size)) isEmpty ifTrue:[
	text grow:(text size - 1)
    ].
    p close.
    calendarView label:text
!

changeYear:aYear
    "change year action from popup menu"

    year := aYear.
    list selection notNil ifTrue:[
	self monthSelection:list selection
    ]
!

otherYear
    |box|

    box := EnterBox new title:'year ?'.
    box action:[:s |
	|n|

	n := Integer readFromString:s.
	(n notNil and:[n ~~ 0]) ifTrue:[
	    self changeYear:n
	]
    ].
    box showAtPointer
! !
