"
 'Non obvious things you can do with blocks' - demonstration
 -----------------------------------------------------------

 fileIn this file, then read the comments in the example methods
 and try them with:
	BlockExamples recursiveExample1
	BlockExamples recursiveExample2
	BlockExamples contextExample1
	BlockExamples closureExample1
	BlockExamples returnExample1
	BlockExamples returnExample2
	BlockExamples returnExample3
	BlockExamples returnExample4
"

Object subclass:#BlockExamples
	 instanceVariableNames:''
	 classVariableNames:''                        
	 poolDictionaries:''                     
	 category:'Examples-Blocks'                       
!

!BlockExamples class methodsFor:'recursive blocks'!

recursiveExample1
    "will not work on PPS before V2.3 and (some?) Smalltalk/V versions"

    |fBlock|

    fBlock := [:n |
	n<= 0 ifTrue:[1]
	      ifFalse:[n * (fBlock value:(n - 1))]].

    ^ (0 to:3) collect:fBlock

    "
     BlockExamples recursiveExample1

     -> should return OrderedCollection(1 1 2 6)"
!

recursiveExample2
    "will not work on PPS before V2.3 and (some?) Smalltalk/V versions"

    |fBlock|

    fBlock := [:n |
	n<= 0 ifTrue:[1]
	      ifFalse:[(fBlock value:(n - 1)) * n]].

    ^ (0 to:3) collect:fBlock.

    "
     BlockExamples recursiveExample2

     -> should return OrderedCollection(1 1 2 6)"
! !

!BlockExamples class methodsFor:'context example'!

contextExample1
    "will not work on PPS before 2.3 and (some?) Smalltalk/V versions"

    | collectionOfBlocks |

    collectionOfBlocks := self contextExample1Helper.
    ^ collectionOfBlocks collect:[:j | j value]

    "
     BlockExamples contextExample1  

     -> should return OrderedCollection(1 2 3 4)"
!

contextExample1Helper
    "helper for contextExample1; return a collection of blocks"

    |c|

    c:=OrderedCollection new. 
    1 to:4 do:[:i|
      c add:[i]     "a block returning i"
    ].
    ^ c             "a collection of blocks returning i', i'', i''' and i''''"
! !

!BlockExamples class methodsFor:'closure example'!

closureExample1
    "this example shows that blocks can live and access method
     locals even when their defining method has already left.
     In this example, the incBlock, decBlock and getBlock access
     a variable hidden in another methods context.
     Notice, that the method which created that variable (as a local variable)
     has already returned; beside through the blocks, there is no access
     possible to that variable. (call it an 'anonymous' variable).
     This example will probably not work in ST/V"

    | blocks incBlock decBlock getBlock|

    "get the blocks (their home method is dead when we are here again)"

    blocks := self closureExample1Helper.
    incBlock := blocks at:1.
    decBlock := blocks at:2.
    getBlock := blocks at:3.

    Transcript show:'variable value == '; showCr:getBlock value.
    Transcript showCr:'increment twice ..'. 
    incBlock value. 
    incBlock value.
    Transcript show:'variable value == '; showCr:getBlock value.

    Transcript showCr:'decrement ..'. 
    decBlock value.
    Transcript show:'variable value == '; showCr:getBlock value.

    "
     BlockExamples closureExample1
    "
!

closureExample1Helper
    "helper for returnExample1; return an array with 3 blocks,
     which access a method local"

    |variable|

    variable := 0.
    ^ Array with:[variable := variable + 1]
	    with:[variable := variable - 1]
	    with:[variable]
! !

!BlockExamples class methodsFor:'return example'!

returnExample1
    "this example shows that a method-return from a block is illegal,
     if the defining method has already left"

    | block |

    "get the block, which contains a return from an already
     left method"

    block := self returnExample1Helper.
    Transcript showCr:'one'.
    block value.
    Transcript showCr:'two'.

    "
     BlockExamples returnExample1
    "
!

returnExample1Helper
    "helper for returnExample1; return a returning block"

    ^ [ 
	Transcript showCr:'in block-1'.
	^ 'blocks value'.
	Transcript showCr:'in block-2'.
      ]
!

returnExample2
    "like returnExample1, this example shows that a method-return from a block 
     is illegal, if the defining method has already left.
     Here we return from a somewhat deeper nested block."

    | block |

    "get the block, which contains a return from an already
     left method. the return is performed in a nested block"

    block := self returnExample2Helper.
    Transcript showCr:'one'.
    block value.
    Transcript showCr:'two'.

    "
     BlockExamples returnExample2
    "
!

returnExample2Helper
    "helper for returnExample2; return a block with a return from
     a nested block"

    ^ [
	Transcript showCr:'in block1-1'.
	[
	  Transcript showCr:'in block2-1'.
	  [
	    Transcript showCr:'in block3-1'.
	    ^ nil.
	    Transcript showCr:'in block3-2'.
	  ] value.
	  Transcript showCr:'in block2-2'
	] value.
	Transcript showCr:'in block1-2'.
    ]
!

returnExample3
    "this example shows how blocks could be used to
     implement some catch&throw mechanism.
     (but now, since Signals and exceptions are available,
      this is no longer a 'good' example)"

    |exitBlock block|

    exitBlock := [^ self].
    block := self returnExample3Helper.
    Transcript showCr:'one'.
    block value:exitBlock.
    Transcript showCr:'two'. "*** not reached ***"

    "
     BlockExamples returnExample3
    "
!

returnExample3b
    "same as returnExample3 - more inline coded"

    |block|

    block := self returnExample3Helper.
    Transcript showCr:'one'.
    block value:[^ self].
    Transcript showCr:'two'. "*** not reached ***"

    "
     BlockExamples returnExample3b
    "
!

returnExample3Helper
    "helper for returnExample3; return a block, which evaluates its
     argument block in a nested block"

    ^ [:exitBlock |
	Transcript showCr:'in block1-1'.
	[:exitBlock |
	  Transcript showCr:'in block2-1'.
	  [:exitBlock |
	    Transcript showCr:'in block3-1'.
	    exitBlock value.
	    Transcript showCr:'in block3-2'.
	  ] value:exitBlock.
	  Transcript showCr:'in block2-2'
	] value:exitBlock.
	Transcript showCr:'in block1-2'.
    ]
!

returnExample4
    "like returnExample3, this shows that a method-return from a block is illegal,
     if the defining method has already left. 
     But since the error is reported using the normal signal mechanism, 
     we can catch this exception and handle it.
     In this case, we simply return from the block."

    | block val |

    "get the block, which contains a return from an already
     left method"

    block := self returnExample1Helper.
    Transcript showCr:'one'.
    Context invalidReturnSignal handle:[:ex |
	"ex parameter will be the to-be-returned value"
	ex proceedWith:(ex parameter).
    ] do:[
	val := block value.
	"*** reached ***"
	Transcript showCr:'value returned:' , val printString.
    ].
    Transcript showCr:'two'.

    "
     BlockExamples returnExample4
    "
! !
