Object subclass:#SignalHandlingExample
	 instanceVariableNames:''
	 classVariableNames:''
	 poolDictionaries:''
	 category:'Examples-Exceptions'!

"
 more examples (see doc/coding/Signal-handling.st) on how to use Signals & Exceptions.
 These are somewhat more tricky ...

 exampleA1 - handle stack overflow - abort execution

 exampleA2 - handle stack overflow - increase stack and retry

 exampleA3 - handle stack overflow - increase stack and continue
"!

!SignalHandlingExample class methodsFor:'examples2'!

exampleA1
    "the handler will just abort"

    Object recursionInterruptSignal handle:[:ex |
	Transcript showCr:'oops - too much recursion'; endEntry.

	ex return.
    ] do:[
	Transcript showCr:'starting 10000 factorial ...'; endEntry.
	10000 factorial.
	Transcript showCr:'done factorial.'; endEntry.
    ].

    Transcript showCr:'thats it'

    "SignalHandlingExample exampleA1"
!

exampleA2
    "the handler will add more stack and restart
     (which works, but is not the best solution - see exampleA3)"

    "make certain, that stack limit is restored, 
     (in case you dont want to wait for the 10000 factorial 
      to be computed, and therefore abort from the debugger ...:-)"

    |originalLimit|

    originalLimit := Processor activeProcess maximumStackSize.

    [
	Object recursionInterruptSignal handle:[:ex |
	    |limit|

	    Transcript showCr:'oops - too much recursion'; endEntry.
	    limit := Processor activeProcess maximumStackSize.
	    Transcript show:'increase stack to '; show:(limit * 2); showCr:' and restart.'; endEntry.
	    Processor activeProcess setMaximumStackSize:(limit * 2).

	    ex restart
	] do:[
	    Transcript showCr:'starting 10000 factorial ...'; endEntry.
	    10000 factorial.
	    Transcript showCr:'done factorial.'; endEntry.
	].
    ] valueNowOrOnUnwindDo:[
	Processor activeProcess setMaximumStackSize:originalLimit
    ].

    Transcript showCr:'thats it'

    "SignalHandlingExample exampleA2"
!

exampleA3
    "the handler will add more stack and continue"

    "make certain, that stack limit is restored, in case
     you dont want to wait for the 10000 factorial to be
     computed, and therefore abort from the debugger ...:-)"

    |originalLimit|

    originalLimit := Processor activeProcess maximumStackSize.
    [
	Object recursionInterruptSignal handle:[:ex |
	    |limit|

	    Transcript showCr:'oops - too much recursion'; endEntry.
	    limit := Processor activeProcess maximumStackSize.
	    Transcript show:'increase stack to '; show:(limit * 2); showCr:' and continue.'; endEntry.
	    Processor activeProcess setMaximumStackSize:(limit * 2).

	    ex proceed
	] do:[
	    Transcript showCr:'starting 10000 factorial ...'; endEntry.
	    10000 factorial.
	    Transcript showCr:'done factorial.'; endEntry.
	].
    ] valueNowOrOnUnwindDo:[
	Processor activeProcess setMaximumStackSize:originalLimit
    ].

    Transcript showCr:'thats it'

    "SignalHandlingExample exampleA3"
! !
